from __future__ import annotations

# MenuCommands

## Main-Menu

### File
FileClose = 57602
FileCloseAll = 33243
FileExit = 32865
FileFontinfo = 33001
FileGenerateFont = 33531
FileNew = 57600
FileOpen = 57601
FilePrint = 57607
FileRevert = 33233
FileSave = 57603
FileSaveAll = 33003
FileSaveAs = 57604

### Edit
EditAppend = 33391
EditCopy = 57634
EditCopyGlyph = 32876
EditCut = 57635
EditDelete = 32871
EditDeselect = 32895
EditDuplicate = 32872
EditFind = 57636
EditInsert = 33183
EditInvertSelectoin = 32875
EditPaste = 57637
EditProperties = 32887
EditRedo = 32854
EditSelectAll = 32873
EditUndo = 32853


## View

### Show-Layers

ViewGrid = 32855
ViewGuidelines = 32856
ViewHints = 32857
ViewMask = 32972
ViewBitmap = 32858
ViewBlues = 32807
ViewMetrics = 33612
ViewPreview = 32862
ViewFontAudit = 32939
ViewEditMask = 33181

ViewCharacterNext = 32864
ViewCharacterPrev = 32863
ViewConnections = 32861
ViewCustomize = 33344
ViewNextPoint = 40143
ViewNodes = 32859
ViewOpenType = 33401
ViewPrevPoint = 40144
ViewQuickZoomin = 33062
ViewQuickZoomout = 33063
ViewRulers = 33335
ViewTemplate = 33400
ViewToolbars = 33364
ViewVectors = 32860
ViewVerticalMetrics = 33398
ViewZoom100 = 33069
ViewZoom125 = -31922
ViewZoom150 = 33071
ViewZoom200 = 33072
ViewZoom25 = 33066
ViewZoom50 = 33067
ViewZoom625 = -31923
ViewZoom75 = 33068
ViewZoomCustom = 33064
ViewZoomFit = 33065
ViewZoomin = 32851
ViewZoomout = 32852

### Window
WindowCascade = 57650
WindowCloseAllGlyphWindows = 32885
WindowManager = 33346
WindowNewGlyph = 32990
WindowNewMetrics = 32991
WindowShowAxis = 32955
WindowShowCollection = 33272
WindowShowFonts = 32946
WindowShowMasters = 32945
WindowShowOpenType = 33202
WindowShowOutput = 163
WindowShowPrimitives = 32977
WindowTileHorz = 57651
WindowTileVert = 57652

### Help
Help = 57670
HelpAbout = 33043
HelpAboutFontLab = 33160
HelpContents = 33044
HelpContext = 57669
HelpIndex = 57666
HelpKeyboardShortcuts = 33348
HelpUsing = 57668


ActionAutohint = 32913
ActionAutohinting = 32816
ActionBlendFonts = 33610
ActionCheckConnection = 32910
ActionConvertToBeziers = 32892
ActionConvertToSpline = 32891
ActionCopy = 32836
ActionCopyglyph = 32839
ActionCut = 32835
ActionDelete = 32812
ActionErase = 32919
ActionExchangeMask = 33617
ActionExpandStroke = 33622
ActionFindSymbol = 32928
ActionFixAll = 3
ActionGenerateInstance = 33279
ActionHMirror = 32908
ActionMaskMetricsToMaster = -32378  # on Windows; on Mac it is 32378
ActionNew = 32884
ActionNodesAtExtremes = 32874
ActionOpenEps = 32880
ActionOptimize = 32911
ActionOverlap = 32846
ActionPaste = 32837
ActionPasteglyph = 32840
ActionPointNext = 33156
ActionPointPrev = 33157
ActionRemoveOverlap = 32846
ActionRemovehints = 32815
ActionReverseAll = 32926
ActionSaveEps = 32881
ActionSelectall = 32838
ActionShiftDown = 33150
ActionShiftDown10 = 33154
ActionShiftLeft = 33147
ActionShiftLeft10 = 33151
ActionShiftRight = 33148
ActionShiftRight10 = 33152
ActionShiftUp = 33149
ActionShiftUp10 = 33153
ActionSnapToGuides = 32890
ActionVMirror = 32909
ActionXOR = 33507
AnchorRename = 40174
BitmapCopy = 33045
BitmapCreate = 32988
BitmapOpen = 33040
BitmapRemove = 32989
BitmapSave = 33041
BitmapTrace = 33042
Bitmapmenu100Scale = 40115
BitmapmenuDelete = 40114
BitmapmenuReset = 40113
ClassAccept = 33333
ClassAdd = 33328
ClassDelete = 33487
ClassMark = 33537
ClassRemove = 33329
ClassSelect = 33536
ClassesMenu = 33559
ClassesOpen = 33557
ClassesReset = 33556
ClassesSave = 33558
CollectDelete = 33278
CollectMark = 33276
CollectSelect = 33275
CollectView = 33525
CollectViewBig = 33524
CollectViewIcon = 33273
CollectViewList = 33274
ExpandButton = 33527
ExpandCommand = 33526
FindCharacter = 33199
FontAddNote = 33264
FontCreateCharacters = 33296
FontEdit = 32981
FontModeCodepages = 33060
FontModeIndex = 33209
FontModeNames = 33208
FontModeRanges = 32974
FontOpenCmap = 33312
FontOpenEditWindow = 32980
FontRemoveUnicode = 33218
FontSaveEncoding = 33176
FontSelectDefault = 33189
FontSelectEncoding = 33138
FontSortByCodepage = 33240
FontSortByName = 33238
FontSortByUnicode = 33239
FontWidth16 = 33055
FontWidth24 = 33056
FontWidth32 = 33057
FontWidth8 = 33058
FontsActivate = 33301
FontsAppend = 33362
FontsDelete = 33302
FontsFontinfo = 33303
FontsMacro = 33309
FontsOpen = 33304
FontsSave = 33305
GlobalguideAscender = 40180
GlobalguideCapsHeight = 40182
GlobalguideDescender = 40181
GlobalguideVisualAscender = 40184
GlobalguideVisualDescender = 40185
GlobalguidexHeight = 40183
GlyphAddAnchor = 33532
GlyphAddNote = 33265
GlyphOptions = 32833
GlyphShowFontWindow = 33078
GlyphbarAnsi = 33323
GlyphbarDecimal = 33319
GlyphbarHex = 33320
GlyphbarIndex = 33326
GlyphbarName = 33318
GlyphbarOctal = 33322
GlyphbarShowMarks = 33327
GlyphbarShowcaption = 33316
GlyphbarUnicode = 33325
GlyphbarWidth = 33324
GuideAlign = 40169
GuideConvertToGlobal = 40172
GuideCreateOrtogonal = 40155
GuidesRemove = 32898
GuidesRemoveH = 32899
GuidesRemoveV = 32900
Help = 57670
HelpAbout = 33043
HelpAboutFontLab = 33160
HelpContents = 33044
HelpContext = 57669
HelpIndex = 57666
HelpKeyboardShortcuts = 33348
HelpUsing = 57668
HintConvertToHint = 40164
HintConvertToLink = 40159
HintDefineStem = 40186
HintReverse = 40165
HintsAddLinkH = 32962
HintsAddLinkV = 32963
HintsAutohint = 32816
HintsAutoreplace = 33079
HintsFromLinks = 32961
HintsRemove = 32815
HintsRemoveH = 32896
HintsRemoveV = 32897
HintsToLinks = 32960
LinkReverse = 40166
LockBlues = 33406
LockComponents = 33410
LockGuides = 33403
LockHints = 33404
LockMask = 33405
LockMetrics = 33408
LockOpentype = 33411
LockOutline = 33402
LockTemplate = 33407
LockVerticalMetrics = 33409
Macro0 = 10140
Macro1 = 10141
Macro2 = 10142
Macro3 = 10143
Macro4 = 10144
Macro5 = 10145
Macro6 = 10146
Macro7 = 10147
Macro8 = 10148
Macro9 = 10149
MacroAssign = 34206
MacroCallEdit = 33192
MacroCallRefresh = 33193
MacroCallRun = 33190
MacroCallStop = 33191
MacroClose = 33490
MacroNew = 33184
MacroOpen = 34188
MacroReload = 33232
MacroRun = 33179
MacroSave = 33185
MacroSaveAs = 33194
MacroStop = 33180
MarkBlue = 33510
MarkCyan = 33513
MarkGreen = 33511
MarkMagenta = 33512
MarkNone = 33508
MarkRed = 33509
MaskClear = 32907
MaskCopy = 32904
MaskExchange = 33617
MaskPaste = 32906
MaskToMaster = 33146
MenuDelete = 40150
MenuProperty = 40151
MetricsAuto = 33105
MetricsClear = 33106
MetricsEditKerning = 32949
MetricsEditMetrics = 32948
MetricsExpandKerning = 33553
MetricsNext = 32952
MetricsOpen = 33100
MetricsPrev = 32951
MetricsPreview = 32950
MetricsPrint = 33102
MetricsQuicksave = 33103
MetricsSave = 33101
MetricsUndo = 33104
MorphmenuReset = 40175
NodeBreak = 40156
NodeConnectionFixed = 40142
NodeConnectionSharp = 40140
NodeConnectionSmooth = 40141
NodeConvert = 40153
NodeDeleteContour = 40161
NodeDuplicateNode = 40190
NodeInsert = 40152
NodeMakeContourFirst = 40160
NodeMakeCorner = 40191
NodeMakeFirst = 40158
NodeNormalize = 40154
NodeRetractBCP = 40194
NodeReverseContour = 40162
NodeSelectContour = 40148
NodeSelectNode = 40149
NodeSubstractContour = 40163
OTAddAnchor = 33313
OTCompile = 33203
OTFeaturesNew = 33222
OTFeaturesOpen = 33210
OTFeaturesSave = 33211
OTKerning = 33486
OTOptions = 33430
OTPrint = 33482
OTRTL = 33363
OTRename = 33521
OTSample = 33206
OTWaterfall = 33429
OperBitmap = 32987
OperCurve = 33046
OperEnvelope = 32994
OperHinting = 32822
OperInterpolate = 32918
OperReverseContours = 32817
OperReversecontours = 32817
OperSet = 33048
OperSetStartpoint = 32818
OperSetnewstartpoint = 32818
OperT1Hinting = 32822
OperTTHinting = 32831
OperTransform = 32830
OperTruetype = 32831
Seacmenu100Scale = 40112
SeacmenuCopyMetrics = 40199
SeacmenuDecompose = 40110
SeacmenuDelete = 40111
SeacmenuHMirror = 40113
SeacmenuOpenComponent = 40197
SeacmenuProperties = 40399
SeacmenuResetShift = 40196
SeacmenuToFront = 40198
SeacmenuToTop = 40198
SeacmenuVMirror = 40114
SelectionAlignpoints = 40193
SelectionToComponent = 40167
SelectionToCurves = 40168
SelectionToLines = 40171
SelectionToOffCurve = 40173
ShowEditLayers = 33242
SnapBlues = 33421
SnapGrid = 33413
SnapGuides = 33415
SnapHints = 33416
SnapMask = 33417
SnapMetrics = 33420
SnapOpenType = 33551
SnapOutline = 33414
SnapTemplate = 33419
SnapVerticalMetrics = 33418
SymbolAddComponent = 33033
SymbolAssignNames = 32929
SymbolCalculateUnicode = 32931
SymbolChooseSymbol = 32927
SymbolClearUnicode = 32932
SymbolCopy = 32839
SymbolCreateSymbol = 33173
SymbolDecompose = 32925
SymbolEditComposite = 33169
SymbolGenerateGlyphs = 33212
SymbolGenerateNames = 33053
SymbolNew = 32884
SymbolOpenEps = 32880
SymbolPaste = 32840
SymbolRenameSymbol = 32930
SymbolSaveEps = 32881
TTHAlign = 40072
TTHConvertHintsToCommands = 40204
TTHDouble = 40074
TTHFinDelta = 40065
TTHGenerateBitmap = 40240
TTHHorizontal = 40071
TTHHorizontalMode = 40071
TTHInterpolate = 40075
TTHMidDelta = 40064
TTHOptions = 40207
TTHRemoveBitmap = 40238
TTHRemoveHorizontalCommands = 40205
TTHRemoveVerticalVommands = 40206
TTHResetFinalDeltas = 40202
TTHResetProgram = 40201
TTHRresetDeltas = 40203
TTHSingle = 40073
TTHVerticalMode = 40070
TTHViewBitmap = 40053
TTHViewCenter = 40052
TTHViewGrid = 40051
TTHViewImage = 40033
TTHViewNumbers = 40054
TTHViewOutline = 40059
TTHViewPpmDown = 40281
TTHViewPpmUp = 40280
TTHViewPreview = 40055
TTHViewProgram = 40050
TemplateClear = 33186
TemplateCopy = 33182
TemplateMaskCopy = 33187
TemplatePaste = 33188
Tool1 = 5000
Tool100 = 40020
Tool2 = 5001
Tool3 = 5002
Tool4 = 5003
Tool5 = 5004
Tool6 = 5005
Tool7 = 5006
Tool8 = 5007
ToolAtmPreview = 40050
ToolAutohint = 40010
ToolCancel = 40002
ToolDraw = 33291
ToolEdit = 32847
ToolErase = 32848
ToolMeter = 32850
ToolOk = 40001
ToolPrimitiveProperties = 40030
ToolPrint = 32834
ToolPython = 33235
ToolPythonList = 33357
ToolVectorPaint = 32903
ToolbarsMacrocall = 33220
ToolsAssignMask = 32973
ToolsCustomize = 33344
ToolsDefineNewAxis = 32967
ToolsEditAxisGraph = 32969
ToolsMacro = 33221
ToolsModifyAxisNames = 32970
ToolsOptions = 32894
ToolsRemoveAxis = 32966
ToolsTransform = 32978
ToolsTransformRange = 32976
VPBrush = 40084
VPBrush1 = 40131
VPBrush2 = 40132
VPBrush3 = 40133
VPBrush4 = 40134
VPBrush5 = 40135
VPBrush6 = 40136
VPBrush7 = 40137
VPBrushAuto = 40080
VPBrushBlack = 40097
VPBrushContour = 40097
VPBrushFlat = 40098
VPBrushRound = 40099
VPBrushStrokes = 4091
VPBrushWhite = 40096
VPBrushWidth1 = 40095
VPBrushWidth2 = 40094
VPBrushWidth3 = 40093
VPEllipse = 40088
VPLine = 40085
VPOptions = 40092
VPPencil = 40089
VPPolygon = 40086
VPRectangle = 40087
VPSelect = 40082
VPText = 40081
WarpmenuSelectPredefined = 40170



from __future__ import annotations

from typing import TYPE_CHECKING

from FL.fake.Base import Copyable
from FL.helpers.FLList import adjust_list

if TYPE_CHECKING:
    from FL.objects.Glyph import Glyph


class KerningPair(Copyable):
    """
    Class to represent kerning pair. This class is Multiple Master-compatible.
    """

    __slots__ = ["_parent", "_key", "_value", "_values"]

    # Constructor

    def __init__(
        self, kerningpair_or_index: KerningPair | int | None = None, value: int = 0
    ) -> None:
        """
        KerningPair()             - generic constructor, creates an empty KerningPair
        KerningPair(KerningPair)  - copy constructor
        KerningPair(index)        - creates a KerningPair to glyph referenced by index
                                    but zero value
        KerningPair(index, value) - creates a KerningPair to glyph referenced by index
                                    but and assigns value

        Args:
            kerningpair_or_index (KerningPair | int | None, optional): _description_. Defaults to None.
            value (int, optional): _description_. Defaults to 0.
        """
        self._parent = None
        self._key = 0
        self._values = [0] * 16  # An orphan pair has values for all 16 possible masters
        if kerningpair_or_index is None:
            self.key = 0
            self.value = 0
        elif isinstance(kerningpair_or_index, KerningPair):
            self._copy_constructor(kerningpair_or_index)
        elif isinstance(kerningpair_or_index, int):
            self.key = kerningpair_or_index
            self.value = value
        else:
            raise TypeError

    @property
    def fake_parent(self) -> None:
        raise AttributeError

    @fake_parent.setter
    def fake_parent(self, value: Glyph | None) -> None:
        self._parent = value
        if self._parent is None:
            # Pad values list to 16 masters
            adjust_list(self._values, 16, 0)  # TODO: Check if the default is 0
        else:
            # Adjust list length to new layers number
            adjust_list(self._values, self._parent.layers_number)

    # Attributes

    @property
    def parent(self) -> Glyph | None:
        """
        KerningPair's parent object, Glyph.
        """
        return self._parent

    @property
    def key(self) -> int:
        """
        index of right glyph of the pair
        """
        return self._key

    @key.setter
    def key(self, value: int) -> None:
        self._key = value

    @property
    def value(self) -> int:
        """
        value of the pair
        """
        return self._values[0]

    @value.setter
    def value(self, value: int) -> None:
        self._values = [value] * len(self._values)

    @property
    def values(self) -> list[int]:
        """
        list of values for each master
        """
        return self._values

    @values.setter
    def values(self, value: list[int]) -> None:
        raise RuntimeError(
            'Attempt to write read only attribute "values" of class KerningPair'
        )
